#!/bin/bash

XML="1.1.xml"
XSD="1.1.xsd"

function die {
  echo "ERROR: $@"
  exit 1
}

function check_file {
  [[ -f "$1" ]] || die "FILE '$1' not found"
}

function strip {
  s=$(cat)
  s=${s## }
  echo ${s%% }
}

check_file "$XSD"

xmllint --version 2> /dev/null\
  || die "xmllint not found. Please install it: sudo apt-get install libxml2-utils"
xsltproc --version > /dev/null\
  || die "xsltproc not found. Please install it: sudo apt-get install xsltproc"

check_file "$XML"
xmllint --noout "$XML" || die "'$XML' is not a valid XML file. Did you modify it?"

xmllint --noout --schema "$XSD" "$XML" 2>&1 \
  || die "The reference XML file '$XML' was not valid with your schema '$XSD'"
echo "Test 0 PASSED: '$XML' is valid for schema '$XSD'"

echo "----------"
VALID="1.1.valid*.xml"
NUM=$(ls $VALID | wc -l | strip)
failed=0
(( NUM >= 2 )) || die "Missing test files? Found only $NUM files matching $VALID"
for i in $VALID; do
  xmllint --noout "$i" 2> /dev/null || die "Bug in the auto-test file '$i'"
  if xmllint --noout --schema "$XSD" "$i" 2>&1; then
    echo -n ""
  else
    echo "ERROR: your schema does not validate test file '$i', but it should!"
    ((failed+=1))
  fi
done
((failed==0)) && echo "Test 1 PASSED: your schema is not too strict."\
    || echo "Test 1 FAILED: your schema is too strict! You had $failed errors."

echo "----------"
INVALID="1.1.invalid*.xml"
NUM=$(ls $INVALID | wc -l | strip)
failed2=0
(( NUM >= 2 )) || die "Missing test files? Found only $NUM files matching $INVALID"
for i in $INVALID; do
  xmllint --noout "$i" 2> /dev/null || die "Bug in the auto-test file '$i'"
  if xmllint --noout --schema "$XSD" "$i" 2> /dev/null; then
    echo "ERROR: your schema validates test file '$i', but it shouldn't!"
    ((failed2+=1))
  else
    echo "$i is invalidated by your schema, as expected."
  fi
done
((failed2==0)) && echo "Test 2 PASSED: your schema is strict enough." \
    || echo "Test 2 FAILED: your schema isn't strict enough. You had $failed2 errors."

if ((failed + failed2 > 0)); then
  echo "FAILED"
  exit 1
fi
echo "----------"
echo "SUCCESS"
